clear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Baseline model
dynare stickyopeninv2 
save simulation0 dr_ ys_ lgy_ lgx_

% Baseline model + Keynesian agents
dynare stickyopeninv3
save simulation1 dr_ ys_ lgy_ lgx_

% Baseline model + Keynesian agents + habits + investment adjustment costs
dynare stickyopeninv4
save simulation2 dr_ ys_ lgy_ lgx_

% number of periods for IRFs.
nperiods = 20;

%%%% Make figure to compare simulations
load simulation0 

shock = [0    %merr
         0    %merr2
         0    %nerr
         0    %nerr2  
         0.01  %sharegerr
         0];  %sharegerr2 
dset = 'f0';

makeirf

load simulation1

shock = [0    %merr
         0    %merr2
         0    %nerr
         0    %nerr2  
         0.01  %sharegerr
         0];  %sharegerr2
dset = 'f1';        

makeirf

load simulation2
[0    %merr
         0    %merr2
         0    %nerr
         0    %nerr2  
         0.01  %sharegerr
         0];  %sharegerr2 dset = 'f1';        
dset = 'f2';        
makeirf



titlelist = char('Output','Capital (next period)','Investment','Consumption','Ex-ante Real Interest Rate','Labor');
legendlist = char('Baseline Model','Baseline with Keynesian Agents');
figlabel = 'Responses to a 1ppt increase in govenment spending as a share of GDP';
makechart(titlelist,legendlist,figlabel,...
          100*[f0_gdp_irf/f0_gdp, f0_kp_irf/f0_kp, f0_ip_irf/f0_ip, f0_cc_irf/f0_cc, f0_rrs_irf, f0_le_irf/f0_le],...
          100*[f1_gdp_irf/f1_gdp, f1_kp_irf/f1_kp, f1_ip_irf/f1_ip, f1_cctot_irf/f1_cctot, f1_rrs_irf, f1_le_irf/f1_le])


titlelist = char('Imports','Exports','RER Production Based','Trade Balance (share of GDP)','RER Consumption Based','Net Foreign Assets (share of gdp)');
makechart(titlelist,legendlist,figlabel,...
          100*[f0_ccm_irf/f0_ccm, f0_ccm2_irf/f0_ccm2, f0_pm_irf/f0_pm, f0_tby_irf, f0_rerpc_irf/f0_rerpc, f0_nfay_irf],...
          100*[f1_ccm_irf/f1_ccm, f1_ccm2_irf/f1_ccm2, f1_pm_irf/f1_pm, f1_tby_irf, f1_rerpc_irf/f1_rerpc, f1_nfay_irf])
   

titlelist = char('Output','Capital (next period)','Investment','Consumption (total)','Government Spending','Keynsian Consumption','Ex-ante Real Interest Rate','Labor');
legendlist = char('Baseline Keynesian','Baseline Keynesian + Real Rigidities');
makechart(titlelist,legendlist,figlabel,...
          100*[f1_gdp_irf/f1_gdp, f1_kp_irf/f1_kp, f1_ip_irf/f1_ip, f1_cctot_irf/f1_cctot,f1_gc_irf/f1_gc, f1_cck_irf/f1_cck, f1_rs_irf, f1_le_irf/f1_le],...
          100*[f2_gdp_irf/f2_gdp, f2_kp_irf/f2_kp, f2_ip_irf/f2_ip, f2_cctot_irf/f2_cctot,f2_gc_irf/f2_gc, f2_cck_irf/f2_cck, f2_rs_irf, f2_le_irf/f2_le])

      
titlelist = char('Inflation (AR)','Nominal Short Rate (AR)','Marginal Cost','Primary Government Deficit (GDP share)','Tax Rate (lump-sum)','Government Debt (GDP share, AR)');
legendlist = char('Baseline Keynesian','Baseline Keynesian + Real Rigidities');
makechart(titlelist,legendlist,figlabel,...
          100*[f1_dp_irf*4, f1_rs_irf*4, f1_sigma_irf/f1_sigma, f1_pgdef2gdp_irf, f1_tr2gdp_irf, f1_gdebt2gdp_irf/4],...
          100*[f2_dp_irf*4, f2_rs_irf*4, f2_sigma_irf/f2_sigma, f2_pgdef2gdp_irf, f2_tr2gdp_irf, f2_gdebt2gdp_irf/4])

titlelist = char('Imports','Exports','Real Exchange Rate (production-based)','Trade Balance (share of GDP)','Relative Price of Imports','Net Foreign Assets (share of gdp)');
makechart(titlelist,legendlist,figlabel,...
          100*[f1_ccm_irf/f1_ccm, f1_ccm2_irf/f1_ccm2, f1_rer_irf/f1_rer, f1_tby_irf, f1_pm_irf/f1_pm, f1_nfay_irf],...
          100*[f2_ccm_irf/f2_ccm, f2_ccm2_irf/f2_ccm2, f2_rer_irf/f2_rer, f2_tby_irf, f2_pm_irf/f2_pm, f2_nfay_irf])
      
          
          
 