// Open economy model with sticky prices, wages, and import prices
// local currency pricing
// variable labor
// rule responds to inflation only
// imports enter both the consumption and the investment bundle
// introduce government spending
// introduce Keynesian households
// introduce habits in consumption

var gc, cc, le, lambdac, q, gdp, ip ,kp, zetap, zetac, rk, n, m, rs, rrs, ccd, ccm, ipd, ipm, dpm, pm, pc, pi, 
    sigma, dpc, dp, dw, wmarkup, nfa, nfay, tby, er, rer, rerpc, sy, ipy, tr, tr2gdp, cck, cctot, gdebt, shareg, gdebt2gdp, pgdef2gdp, 
    gc2, cc2, le2, lambdac2, q2, gdp2, ip2 ,kp2, zeta2, zetac2, rk2, n2, m2, ccd2, ccm2, ipd2, ipm2, dpm2, pm2, pc2, pi2, 
    sigma2, dpc2, dp2, dw2, wmarkup2, rs2, rrs2, tby2, rer2, rerpc2 tr2, cck2, cctot2, gdebt2, shareg2 ;

varexo nerr, merr, nerr2, merr2, sharegerr, sharegerr2;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// % define parameters here
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parameters beta_p, chi_p, le_p, delta_p, psik_p, psii_p, psic_p,
           alpha_p, s_p, chi0_p, rhon_p, rhom_p, rho_shareg_p, 
           rs_dp_p, rs_gdp_p, omegac_p, omegai_p, rhoc, nfa_p, dw_p, dp_p, sigma_p, 
           lambdac_p, zetac_p, wmarkup_p, kappap_p, kappam_p, kappaw_p, gdp_p, shareg_p, gc_p,
           sharekey_p, cctot_p, cck_p, tr_p, trl_p, trd_p, tauw_p, taup_p,xip_p, xim_p, xiw_p, thetaw_p;
    

// Discount factor  (default .99)
beta_p = 0.99;

// Frisch Elasticity of labor supply (2/chi_p);
chi_p = 10;

// Steady state labor supply
le_p = 0.5;

// depreciation rate (default 0.025)
delta_p = 0.025;
 
// capital share of gdp
alpha_p = 0.3;

// Calvo parameter for domestic
// intermediate prices
xip_p = .75;

// Calvo parameter for export prices
xim_p = .75;

// calvo parameter for wages
xiw_p = .75;

// government spending share of GDP
shareg_p = .2;

// real marginal cost in SS
sigma_p = 1;

// wage markup, in steady state
thetaw_p = .10;

// wage subsidy
tauw_p = thetaw_p;

// production subsidy
taup_p = 0.10;

// coefficient on marginal cost in inflation equation
kappap_p = (1-beta_p*xip_p)*(1-xip_p)/xip_p;

// coefficient on marginal cost in export price inflation equation
kappam_p = (1-beta_p*xim_p)*(1-xim_p)/xim_p;

// coefficient on mrs in the wage inflation equation
kappaw_p = (1-beta_p*xiw_p)*(1-xiw_p)/xiw_p /( 1 + chi_p*le_p/(1-le_p)*(1+thetaw_p)/thetaw_p )

// price inflation
dp_p = 1;

// wage inflation
dw_p = 1;

// rental rate for capital in ss
rk_p = 1/beta_p -1 +delta_p;

// wage rate in ss
zetac_p = (1-alpha_p)*(alpha_p/rk_p)^(alpha_p/(1-alpha_p));

// gdp in ss
gdp_p = (alpha_p/rk_p)^(alpha_p/(1-alpha_p))*le_p;

// saving rate in ss
s_p = (alpha_p/rk_p)*(delta_p);  

// government spending in SS
gc_p = shareg_p *gdp_p;

tr_p =  tauw_p*zetac_p*le_p + gc_p + taup_p*rk_p*s_p/delta_p*gdp_p;

// total consumption in ss
cctot_p = (1-s_p-shareg_p)*gdp_p;

// consumption of keynesian agents 
cck_p = (1+tauw_p) * zetac_p * le_p - tr_p;

// share of Keynesian agents
sharekey_p = 0.5 ;

// consumption of optimizers
cc_p = 1/(1-sharekey_p)*(cctot_p -sharekey_p*cck_p);

// lambda in ss
lambdac_p = 1/cc_p;

// parameter determining labor supply in SS
chi0_p = (1-le_p)^chi_p * lambdac_p * zetac_p

// wmarkup
wmarkup_p = (((1-le_p)^(-chi_p))/lambdac_p) / zetac_p

// weight of domestic goods in consumption
// determines home bias

omegac_p = 0.88;
omegai_p = 0.88;

// import price elasticity of substitution
el = 2
rhoc = 1/(el-1);

// parameters governing the interest rate rule
rs_dp_p = 10;
rs_gdp_p = 0;

// parameters governing the tax rate reaction function
trl_p = 0.01;
trd_p = 0.01;

psik_p = 0;
psic_p = 0.8;
psii_p = 3;

// AR1 parameters for shock processes
rhon_p = 0.95;    
rhom_p = 0.95;
rho_shareg_p = 0.99;
     
// stationarity-inducing parameter
nfa_p = 0.001;
  
model;
///////////////////////////////////////////////////////////////////////////
// Country 1
///////////////////////////////////////////////////////////////////////////

// 1.  FOC with respect to consumption for optimizing agents
(1-psic_p)/(cc-psic_p*cc(-1)) - lambdac=0;   

// Consumption rule for Keynesian agents
pc * cck = (1+tauw_p) * zetap * le - tr; 

// government budgent constraint
gdebt/(1+rs)*dp(1) = gdebt(-1) - tr + tauw_p*zetap*le + gc + taup_p*rk*kp;

// government debt as a share of GDP
gdebt2gdp = gdebt(-1)/gdp;

// primary deficit as a share of GDP
pgdef2gdp = (- tr + tauw_p*zetap*le + gc + taup_p*rk*kp)/gdp;

// definition of aggregate consumption
cctot = (1-sharekey_p)*cc + sharekey_p*cck;

// net tax reaction function (tr is net transfers)
tr = tr(-1) + trl_p *(gdebt/gdp) + trd_p *(gdebt/gdp-gdebt(-1)/gdp(-1));

// tax rate 
tr2gdp = tr/gdp;

// 2. Definition of production real wage
zetap/zetap(-1) = dw/dp;

// 3. Definition of consumption real wage
zetap = zetac*pc;

// 4. Wage inflation equation
log(dw) - log(dw_p) = beta_p* ( log(dw(1)) - log(dw_p) ) 
                      +kappaw_p*(log(wmarkup)-log(wmarkup_p));

// 5. Wage markup
log(wmarkup)-log(wmarkup_p) = chi_p*le_p/(1-le_p)*(log(le)-log(le_p)) 
                              - (log(lambdac)-log(lambdac_p))
                              - (log(zetac)-log(zetac_p));

// 6. Lambda Marginal Utility of consumption
q = 1+ psik_p*(ip/kp(-1)-delta_p) + psii_p*(ip/ip(-1)-1)
     - psii_p*dp(1)/(1+rs)*(ip(1)/ip-1)
     - psii_p/2*dp(1)/(1+rs)*(ip(1)/ip-1)^2;

// 7. gamma shadow price of capital
q = dp(1)/(1+rs)*((1-delta_p)*q(1) + rk(1) - psik_p/2*(ip(1)/kp - delta_p)^2 + psik_p*ip(1)/kp*(ip(1)/kp-delta_p));

// 8. Aggregate Output
gdp = (kp(-1)*exp(m))^(alpha_p)*(le*exp(n))^(1-alpha_p);

// 9. resource constraint
gdp = gc + ccd + ipd + ccm2+ ipm2;

// 10. definition of consumption aggregator (pins down ccd)
cctot = (omegac_p^(rhoc/(1+rhoc))*ccd^(1/(1+rhoc))+(1-omegac_p)^(rhoc/(1+rhoc))*ccm^(1/(1+rhoc)))^(1+rhoc);

// 11. import demand
ccm = (1-omegac_p)*(pm/pc)^(-(1+rhoc)/rhoc) * cctot;

// 12. consumption prices
pc = (omegac_p+(1-omegac_p)*pm^(-1/rhoc))^(-rhoc);


// 13. definition of investment aggregator (pins down ccd)
ip = (omegai_p^(rhoc/(1+rhoc))*ipd^(1/(1+rhoc))+(1-omegai_p)^(rhoc/(1+rhoc))*ipm^(1/(1+rhoc)))^(1+rhoc);

// 14. investment import demand
ipm = (1-omegai_p)*(pm/pi)^(-(1+rhoc)/rhoc) * ip;

// 15. investment prices
pi = (omegai_p+(1-omegai_p)*pm^(-1/rhoc))^(-rhoc);


// 16. FOC for export prices
log(dpm2) - log(dp_p) = beta_p * (log(dpm2(1)) - log(dp_p)) + kappam_p*(log(sigma) - log(sigma_p) - log(rer)-log(pm2)); 

dpm/dp = pm/pm(-1);

// 17. consumption price inflation
dpc = pc/pc(-1);

// 18. pricing equation
log(dp) - log(dp_p) = beta_p * (log(dp(1)) - log(dp_p)) + kappap_p*(log(sigma) - log(sigma_p)); 

// 19. real nfa
rer*nfa*dp2(1)/(1+rs2)/exp(-nfa_p/2*(rer*nfa/gdp)) = rer*nfa(-1) + ccm2+ipm2-pm*(ccm+ipm) ;

// 20. nfa to gdp
nfay = rer*nfa/gdp;

// 21. Trade Balance, share of GDP
tby = (ccm2-pm*ccm)/gdp;

// 22. Saving rate
sy = 1-shareg -cctot*pc/gdp;

// 23. Investment rate
ipy = ip/gdp;

// 24. UIP equation
1/(1+rs)*er(1)/er = 1/(1+rs2)/exp(-nfa_p/2*(rer*nfa/gdp)) ;

// 25. 
er/er(-1) = rer/rer(-1)*dp2/dp;

// 26. This is the home risk-free rate.
dpc(1)/(1+rs) = beta_p *lambdac(1)/lambdac;

// 27. interest rate reaction function
rs = 1/beta_p -1 + rs_dp_p * (dp-dp_p)
                 + rs_gdp_p * (log(gdp) - log(gdp(-1)));

// 28. real ex-ante rate
1/(1+rrs) = dpc(1)/(1+rs);

// 29. Consumption-based real exchange rate
rerpc = rer*pc2/pc;

// 30. Capital
kp = (1-delta_p)*kp(-1) + ip;

// 31. Rental Rate for Labor, FOC with respect to labor
sigma = zetap/((1-alpha_p)*(gdp/(le*exp(n)))*exp(n));

// 32. Rental Rate for Capital
sigma = rk/(alpha_p*(gdp/(kp(-1)*exp(m))) * exp(m));

// government spending
gc = shareg * gdp;

// 33. Labor-augmenting technology shock
n = rhon_p*n(-1) + nerr;

// 34. Capital Augmenting Technology Shock
m = rhom_p*m(-1) + merr;

// Shock to the share of government spending
shareg - shareg_p = rho_shareg_p * (shareg(-1)-shareg_p) + sharegerr;


///////////////////////////////////////////////////////////////////////////
// Country 2
///////////////////////////////////////////////////////////////////////////

// 35. FOC with respect to consumption
(1-psic_p)/(cc2-psic_p*cc2(-1)) - lambdac2=0;   

pc2 * cck2 = (1+tauw_p) * zeta2 * le2 - tr2; 

gdebt2/(1+rs2)*dp2(1) = gdebt2(-1) - tr2 + tauw_p*zeta2*le2 + gc2 + taup_p*rk2*kp2;

cctot2 = (1-sharekey_p)*cc2 + sharekey_p*cck2;

tr2 = tr2(-1) + trl_p *(gdebt2/gdp2) + trd_p *(gdebt2/gdp2-gdebt2(-1)/gdp2(-1));

// 36. Definition of real wage
zeta2/zeta2(-1) = dw2/dp2;

// 37. Definition of consumption real wage
zeta2 = zetac2*pc2;

// 38. Wage inflation equation
log(dw2) - log(dw_p) = beta_p * ( log(dw2(1)) - log(dw_p)) 
                      +kappaw_p*(log(wmarkup2)-log(wmarkup_p));

// 39. Wage markup
log(wmarkup2)-log(wmarkup_p) = chi_p*le_p/(1-le_p)*(log(le2)-log(le_p)) 
                              - (log(lambdac2)-log(lambdac_p))
                              - (log(zetac2)-log(zetac_p));

// 40. Lambda Marginal Utility of consumption
q2 = 1+ psik_p*(ip2/kp2(-1)-delta_p) + psii_p*(ip2/ip2(-1)-1)
     - psii_p*dp2(1)/(1+rs2) *(ip2(1)/ip2-1)
     - psii_p/2*dp2(1)/(1+rs2)*(ip2(1)/ip2-1)^2;


// 41. gamma shadow price of capital
q2 = dp2(1)/(1+rs2)*((1-delta_p)*q2(1) + rk2(1) - psik_p/2*(ip2(1)/kp2 - delta_p)^2 + psik_p*ip2(1)/kp2*(ip2(1)/kp2-delta_p));

// 42. Aggregate Output
gdp2 = (kp2(-1)*exp(m2))^(alpha_p)*(le2*exp(n2))^(1-alpha_p);

// 43. resource constraint
gdp2= gc2+ipd2 + ccd2 + ccm +ipm;

// 44. definition of consumption aggregator (pins down ccd)
cctot2 = (omegac_p^(rhoc/(1+rhoc))*ccd2^(1/(1+rhoc))+(1-omegac_p)^(rhoc/(1+rhoc))*ccm2^(1/(1+rhoc)))^(1+rhoc);

// 40. import demand
ccm2 = (1-omegac_p)*(pm2/pc2)^(-(1+rhoc)/rhoc) * cctot2;

// 41. 
pc2 = (omegac_p+(1-omegac_p)*pm2^(-1/rhoc))^(-rhoc);

// 10. definition of investment aggregator (pins down ccd)
ip2 = (omegai_p^(rhoc/(1+rhoc))*ipd2^(1/(1+rhoc))+(1-omegai_p)^(rhoc/(1+rhoc))*ipm2^(1/(1+rhoc)))^(1+rhoc);

// 11. investment import demand
ipm2 = (1-omegai_p)*(pm2/pi2)^(-(1+rhoc)/rhoc) * ip2;

// 12. investment prices
pi2 = (omegai_p+(1-omegai_p)*pm2^(-1/rhoc))^(-rhoc);


// 13. FOC for export prices
log(dpm) - log(dp_p) = beta_p * (log(dpm(1)) - log(dp_p)) + kappam_p*(log(sigma2) - log(sigma_p) - log(rer2)-log(pm)); 

// definition of import prices
dpm2/dp2 = pm2/pm2(-1);

rer2 = 1/rer;

// consumption price inflation
dpc2 = pc2/pc2(-1);

// pricing equation
log(dp2) - log(dp_p) = beta_p * (log(dp2(1)) - log(dp_p)) + kappap_p*(log(sigma2) - log(sigma_p)); 

// interest rate reaction function
rs2 = 1/beta_p -1 + rs_dp_p * (dp2 - dp_p)
                  + rs_gdp_p * (log(gdp2) - log(gdp2(-1)));


1/(1+rrs2) = dpc2(1)/(1+rs2);

// Trade Balance (share of GDP)
tby2 = (ccm-pm2*ccm2)/gdp2;

dpc2(1)/(1+rs2) = beta_p *lambdac2(1)/lambdac2;

// Capital
kp2 = (1-delta_p)*kp2(-1) + ip2;

// Rental Rate for Labor, FOC with respect to labor
sigma2 = zeta2/((1-alpha_p)*(gdp2/(le2*exp(n2)))*exp(n2));

// Rental Rate for Capital
sigma2 = rk2/(alpha_p*(gdp2/(kp2(-1)*exp(m2)))* exp(m2));

// Consumption-based real exchange rate
rerpc2 = 1/rerpc;

// government spending
gc2 = shareg2*gdp2;

// Labor-augmenting technology shock
n2 = rhon_p*n2(-1) + nerr2;

// Capital Augmenting Technology Shock
m2 = rhom_p*m2(-1) + merr2;

// Shock to the share of Government Spending
shareg2 - shareg_p = rho_shareg_p * (shareg2(-1)-shareg_p) + sharegerr2; 


end;


initval;
///////////////////////////////////////////////////////////////////////////
// Steady state for  country 1
///////////////////////////////////////////////////////////////////////////

gdebt = 0;

tr = tr_p;

dw = dw_p;

dp = dp_p;

dpc = dp_p;

dpm = dp_p;

wmarkup = wmarkup_p;

sigma = sigma_p;

// net foreign assets
nfa =0;

// net foreign assets to gdp
nfay = 0;

// consumption-based real exchange rate
rerpc = 1;

// production-based real exchange rate
rer=1;

er=1;

// saving rate
sy = s_p;

// investment rate
ipy = s_p;

// short term real rate
rs = 1/beta_p -1;

rrs = rs;

// Labor
le = le_p;

// Output
gdp = gdp_p;

// Investment
ip = (s_p * gdp);

// Consumption
cc = cc_p;

cck = cck_p;

cctot = cctot_p;

// domestic part of consumption
ccd = omegac_p*cctot_p;

ccm = (1-omegac_p)*cctot_p;

ipd = omegai_p * ip;

ipm = (1-omegai_p)*ip;

pm = 1;

pc = 1;

pi = 1;

// Capital 
kp = ip/delta_p;

// Rental rate for capital
rk = rk_p;

// Rental rate for labor
zetap = zetac_p;

zetac = zetac_p;

// Marginal utility of consumption
lambdac = 1/cc;

// tobin's marginal q
q = 1;

// government spending
gc = shareg_p * gdp;
shareg = shareg_p;
gdebt2gdp = gdebt/gdp;
pgdef2gdp = 0;
tr2gdp = tr/gdp;

// Labor-augmenting technology
n=0;

// Capital-augmenting technology
m=0;

///////////////////////////////////////////////////////////////////////////
// Steady state for country 2
///////////////////////////////////////////////////////////////////////////
tr2 = tr_p;

gdebt2 = 0;

dw2 = dw_p;

dp2 = dp_p;

dpc2 = dp_p;

dpm2 = dp_p;

wmarkup2 = wmarkup_p;

sigma2= sigma_p;

rs2 = 1/beta_p -1;

rrs2 = rs2;

// Consumption-based real exchange rate
rerpc2 = 1;

rer2 = 1;

// Labor
le2 = le_p;

// Output
gdp2 = gdp_p;

// Investment
ip2 = (s_p * gdp);

// Consumption
cc2 = cc_p;

cctot2 = cctot_p;

cck2 = cck_p;
// domestic part of consumption
ccd2 = omegac_p*cctot_p;

ccm2 = (1-omegac_p)*cctot_p;

ipd2 = omegai_p*ip2;

ipm2 = (1-omegai_p)*ip2;

pm2 = 1;

pc2 = 1;

pi2 = 1;

// Capital 
kp2 = ip/delta_p;

// Rental rate for capital
rk2 = (delta_p - 1 + 1/beta_p);

// Rental rate for labor
zetac2 = zetac_p;
zeta2 = zetac_p;

// Marginal utility of consumption
lambdac2 = lambdac_p;

// GAMMA: Marginal utility of capital
q2=1;

// government spending
gc2 = shareg_p * gdp2;
shareg2 = shareg_p;

// Labor-augmenting technology
n2=0;

// Capital-augmenting technology
m2=0;



end;

shocks;
var nerr; stderr .01;
var merr; stderr .01;
var sharegerr; stderr .01;
var merr2; stderr .01;
var nerr2; stderr .01;
var sharegerr2; stderr .01;
end;

check;

stoch_simul(order=1,nocorr,nomoments,irf=0);
